(ns puppetlabs.activity.http.middleware
  (:require [clojure.tools.logging :as log]
            [puppetlabs.activity.http.util :as http-util]
            [puppetlabs.rbac.http.middleware :as rbac]
            [slingshot.slingshot :refer [try+]]))

(defn- wrap-errors
  [handler]
  (fn [request]
    (try+ (handler request)
          (catch AssertionError e
            (rbac/build-response request 400 {:kind    "bad-request"
                                              :msg     "Bad request."
                                              :details {:url (request :uri), :params (request :params)}}))
          (catch [:kind :puppetlabs.activity/not-found] {}
            (rbac/build-response request 404 {:kind    "not-found"
                                              :msg     "The resource could not be found."
                                              :details (request :uri)})))))

(defn wrap-errors-with-explanations
  [handler login-path]
  "The standard set of middleware wrappers to use"
  (-> handler
      wrap-errors
      (rbac/wrap-authn-errors login-path)
      rbac/wrap-authz-errors
      rbac/wrap-catchall-500))